@@ -35,6 +35,10 @@
 #' Data can include baseline covariates. Methods for time-dependent covariates
 #'  are not currently available.
 #'
+#' If input lag is provided as a numeric vector or as a column of the data,
+#'   lag values should be set to Inf or NA for participants that do not reach 
+#'   full efficacy.
+#'
 #' There are 3 possible weighting options, the selection of which is determined
 #'  by the combination of model inputs. 
 #'   \describe{
@@ -52,8 +56,7 @@
 #'  tools ve() and plot(). Specifically, "gFunc" is an integer object
 #'  specifying the model selected for the infection rate (based on input gFunc);
 #'  "v", the knots or cut-offs to be used by gFunc (input v);
-#'  "maxTau", the maximum vaccination time included in the analysis; 
-#'  "lag", the lag time between the initial vaccine dose and full efficacy;
+#'  "maxTime", the maximum time since full efficacy included in the analysis; 
 #'  "phaseType", =1 only unblinded phase, =2, only blinded phase, =3 both phases,
 #'  and "wgtType", =0 no weighting, =1 censor weighting, =2 full weighting.
 #'
@@ -69,6 +72,12 @@
 #'   used to estimate theta values if possible. If ="b", only the blinded
 #'   phase will be used. If ="u", only the unblinded phase will be used.
 #'
+#' @param cutoff A numeric object. The minimum proportion of infections that
+#'   must occur during a phase to be included in the analysis. The default
+#'   is 0.1 (10 %). Specifically -- if < 10% of the infections occur during the 
+#'   blinded (unblinded) phase, only the unblinded (blinded) phase will be 
+#'   included in the analysis.
+#'
 #' @param modelEntry A formula object or NULL. The coxph model for entry times.
 #'   The LHS is set as the appropriate Surv() object internally. If a LHS
 #'   is provided, it is ignored. If NULL, inputs modelPsi, modelUnblind0,
@@ -107,10 +116,14 @@
 #' @param variant An integer object. The variant for the analysis. If 0,
 #'   all variants are included in the analysis.
 #'
-#' @param lag A scalar numeric object. The lag time between 
-#'   the initial vaccine dose and full efficacy for all participants. 
-#'   The default value is 6 weeks (42 days) -- NOTE this assumes that the
-#'   data are on the scale of weeks.
+#' @param lag A scalar numeric, numeric vector, or character object. The lag 
+#'   time(s) between the initial vaccine dose and full efficacy. If a scalar,
+#'   the provided lag time applies to all participants. If a numeric vector,
+#'   the vector contains the individual specific lag time for each participant
+#'   (see details for further information).
+#'   If character, the column header of the data containing the lag times.
+#'   The default value is a scalar value of 6 weeks (42 days) -- NOTE this 
+#'   assumes that the data are on the scale of weeks. 
 #'
 #' @param v A numeric vector object. 
 #'   The knots or cut-offs to be used for input gFunc.
@@ -163,7 +176,7 @@
 #'   \item{theta}{A vector object containing the estimated theta parameters.}
 #'   \item{cov}{The covariance estimated using the sandwich estimator.}
 #'   \item{SE}{The standard error estimated using the sandwich estimator.}
-#'   and attributes  "gFunc", "maxTau", "lag", "v", "phaseType", and "wgtType", 
+#'   and attributes  "gFunc", "maxTime", "v", "phaseType", and "wgtType", 
 #'   which store
 #'   details of the original analysis that are required for post-processing
 #'   convenience functions ve() and plot(). See details for further
@@ -221,6 +234,7 @@ veWaningVariant <- function(data,
                             L, 
                             ..., 
                             phase = "ub",
+                            cutoff = 0.1,
                             lag = 6.0,
                             modelEntry = NULL,
                             modelUnblind0 = NULL,
@@ -241,6 +255,17 @@ veWaningVariant <- function(data,
                             Psi = "Psi",
                             Delta = "Delta") {
 
+  # data must be a data.frame or matrix with column headers
+  if (is.matrix(x = data)) {
+    if (is.null(x = colnames(x = data))) {
+      stop("data must be a data.frame object", call. = FALSE)
+    }
+    data <- as.data.frame(x = data)
+  }
+  if (!is.data.frame(x = data)) {
+      stop("data must be a data.frame object", call. = FALSE)
+  }
+
   # ensure the provided column names are characters
   if (any(!is.character(x = txName) ||
           !is.character(x = U) ||
@@ -262,40 +287,31 @@ veWaningVariant <- function(data,
                "Psi" = Psi,
                "Delta" = Delta)
 
-  # generate random name for lag variable
-  #
-  # with the use of a single lag time, this is unnecessary, but is
-  # kept in anticipation of allowing for individual dependent lag times
-  # in the future
-  while (TRUE) {
-    lagName = paste(sample(x = LETTERS, size = 10, replace = TRUE), 
-                    collapse = "")
-    if (lagName %in% colnames(x = data)) next
-    break
-  }
-
-  # data must be a data.frame or matrix with column headers
-  if (is.matrix(x = data)) {
-    if (is.null(x = colnames(x = data))) {
-      stop("data must be a data.frame object", call. = FALSE)
+  # generate random name for lag variable if not provided
+  if (is.character(x = lag)) {
+    dObj$lag <- lag
+  } else {
+    while (TRUE) {
+      lagName = paste(sample(x = LETTERS, size = 10, replace = TRUE), 
+                      collapse = "")
+      if (lagName %in% colnames(x = data)) next
+      break
     }
-    data <- as.data.frame(x = data)
+    dObj$lag <- lagName
   }
-  if (!is.data.frame(x = data)) {
-      stop("data must be a data.frame object", call. = FALSE)
-  }
-
-  dObj$lag <- lagName
 
   if (is.numeric(x = lag)) {
     # This catches both integer and numeric objects
-    if (length(x = lag) == 1L) {
+    if ({length(x = lag) == 1L} || {length(x = lag) == nrow(x = data)}) {
       data[,dObj$lag] <- lag
     } else {
-      stop("when provided, lag must be scalar", call. = FALSE)
+      stop("when provided as a numeric, lag must be a scalar ",
+           "or a vector of length nrow(data)", 
+           call. = FALSE)
     }
-  } else {
-    stop("lag must be scalar", call. = FALSE)
+  } else if (!is.character(x = lag)) {
+    stop("lag must be numeric scalar, numeric vector, or character object", 
+         call. = FALSE)
   }
 
   # extracting baseline covariates included in models
@@ -322,6 +338,7 @@ veWaningVariant <- function(data,
 
   # verify phases to be included in analysis and extract infection times
   phaseInfo <- .verifyPhase(phase = phase, 
+                            cutoff = cutoff,
                             data = data, 
                             subset = gFunc$group, 
                             dObj = dObj)
@@ -376,15 +393,17 @@ veWaningVariant <- function(data,
     allTau <- data[,dObj$R] * {data[,dObj$Gam] == 1L}
   }
 
-  maxTau <- max(allTau)
-  if (maxTau < lag) {
-    message("no participant has been vaccinated longer than the lag time")
-    maxTau <- NULL
+  # maxTime is used in post-processing as the upper-bound of the range of
+  # allowed times. Switching to "time since full efficacy" or (tau - lag)
+  # means this value needs to be shifted down.
+  maxTime <- max(allTau - data[,dObj$lag])
+  if (maxTime < 0.0) {
+    message("no participant has reached full efficacy")
+    maxTime <- NULL
   }
 
   attr(out, "gFunc") <- gFunc$gFunc
-  attr(out, "maxTau") <- maxTau
-  attr(out, "lag") <- lag
+  attr(out, "maxTime") <- maxTime
   attr(out, 'v') <- gFunc$v
   attr(out, 'phaseType') <- phaseInfo$phaseType
   attr(out, 'wgtType') <- modelInfo$wgtType
